//Written for www.HomoFaciens.de
//Program calculates field line pattern of a positive and a negative charge.
//Coded by Norbert Heinz in 2014
//
//The code was written to create some animation sequences for a video about electrostatics:
//http://www.homofaciens.de/technics-charge_en_navion.htm
//This is a fraction of the total code doing nothing but draw the field lines of a positive and a negative charge.
//
//Created and tested with Ubuntu Linux:
//1.) Get a Live and installations DVD of Ubuntu at www.ubuntu.com (if not already installed on your hard disk)
//2.) Open Terminal and install additional components:
//    sudo apt-get install freeglut3 freeglut3-dev
//    To create picture sequences:
//    sudo apt-get install imagemagick
//
//3.) Compile with: gcc FieldLines.c -lglut -lGLU -lGL -o FieldLines
//4.) Run with: ./FieldLines
//
//
//This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free
//Software Foundation version 3 of the License.
//
//This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
//
 
#include <stdio.h>
#include <stdlib.h>
#include <GL/glx.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <X11/extensions/xf86vmode.h>
#include <X11/keysym.h>
#include <math.h>
#include <string.h>

#define CHARACTERS 256            //Number of characters
#define PI 3.1415927f
#define PATHMAX 20


GLfloat SchriftFarbe[6];
GLfloat HGFarbe[3]={0.0f, 0.0f, 0.0f};


int gridX = 64;
int gridY = 32;
GLfloat GRIDDIVISOR = 5.0f;
GLfloat SCALE = 0.1f;

int slowDownMS = 0;
int animationPause = 0;
int temp = 0;
int animation = 0;
int savePics = 0;
int picNumber = 0;
int firstStart = 10;
int paintLastPic = 0;
GLfloat minDistance = 0.35f;
GLfloat particleCharge = 1.0f, testCharge = 1.0f;
GLfloat coordScale;
GLfloat xMax = 8.0, yMax;
GLfloat fieldLineEndUp = 0.1f;
GLfloat fieldLineEndDown = 0.1f;
GLfloat particles[1000][7];
GLfloat path[20][30000][3];
int pathEnd[20];
int pathColor[20];
GLfloat potentialPoints[10][3];
double k1 = 1.0 / (4.0 * PI * 8.85419E-12);
double alpha;
double doubleTemp = 0.0;
GLfloat GLfloatTemp = 0.0f;
int particleNumber = 1;
int potentialEnd = 1;
GLfloat scalePlaneHeigth = 200.0f;
GLfloat planeOffset = 0.0f;
GLfloat moveAlpha = 0.0f;
GLfloat moveBeta = 0.0f;
GLfloat minValue, maxValue, minValue1, maxValue1, minValue2, maxValue2;
GLfloat totalCharge = 0.0f;
GLfloat minVectorDistance = 0.15f;
GLfloat pathWidth[PATHMAX];
GLfloat potentialSphereRadius = 0.16f;
GLfloat protonRadius = 0.85f;
GLfloat particleSpeed = 0.9f;
GLfloat dipoleRadius = 0.05f;

int enableLight1 = 1, enableLight2 = 1, enableLight3 = 1;
int enablePotential = 0;
int enablePotentialPoints = 0;
int enablePotentialDiff = 0;
int enablePotentialPlane = 0;
int enableParticles = 0;
int enableMinDisc = 0;
int enableFieldVectors = 0;
int enableLablesFieldVectors = 1;
int enableGrid = 1;
int enableSecondEquipotential = 1;
int enablePotentialSphere = 0;
int startPicNumber = 0;
int enableTransLucentSphere = 0;

int pauseFrames = 0;
GLfloat potentialSphereX = 0.0f, potentialSphereY = 0.0f, potentialSphereZ = 0.0f;

GLuint base;            /* base display list for the font */

GLfloat planeColor[] = {1.0, 1.0, 1.0}; 
GLfloat axeXColor[] = {1.0, 0.0, 1.0}; 
GLfloat axeYColor[] = {0.0, 1.0, 1.0}; 
GLfloat axeZColor[] = {1.0, 1.0, 0.0}; 


GLfloat redMaterial[] = {1.0, 0.0, 0.0}; 
GLfloat blueMaterial[] = {0.0, 0.0, 1.0};
GLfloat greenMaterial[] = {0.0, 1.0, 0.0}; 
GLfloat orangeMaterial[] = {1.0, 0.0, 1.0}; 

GLfloat lightWhite1[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightRed1[] = { 1.0f, 0.0f, 0.0f, 1.0f };
GLfloat lightRed2[] = { 0.7f, 0.0f, 0.0f, 0.50f };
GLfloat lightRed3[] = { 1.0f, 0.6f, 0.6f, 0.50f };
GLfloat lightGreen1[] = { 0.0f, 1.0f, 0.0f, 1.0f };
GLfloat lightBlue1[] = { 0.0f, 0.7f, 1.0f, 1.0f };
GLfloat lightBlue2[] = { 0.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightBlue3[] = { 0.0f, 0.4f, 1.0f, 1.0f };
GLfloat lightOrange1[] = { 1.0f, 0.7f, 0.0f, 1.0f };
GLfloat lightOrange2[] = { 0.7f, 0.5f, 0.0f, 0.50f };
GLfloat lightSpecularOrange1[] = { 0.5f, 0.35f, 0.0f, 0.3f };

GLfloat lightTemp[] = { 1.0f, 0.0f, 0.0f, 1.0f };

GLfloat lightPosition0[] = { -5.0f, -2.0f, 5.0f, 1.0f };

GLfloat specular1[] = { 1.0f, 1.0f, 1.0f, 1.0f}; 
GLfloat lightSpot1[] = { 0.3f, 0.3f, 0.3f, 0.3f}; 
GLfloat lightAmbient1[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightDiffuse1[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightPosition1[] = { -5.0f, 2.0f, 5.0f, 1.0f };

GLfloat specular2[] = { 1.0f, 1.0f, 1.0f, 0.3f}; 
GLfloat lightSpot2[] = { 0.3f, 0.3f, 0.3f, 0.3f}; 
GLfloat lightAmbient2[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightDiffuse2[] = { 0.3f, 0.3f, 0.3f, 0.5f };
GLfloat lightPosition2[] = { 0.0f, 2.0f, 10.0f, 1.0f };

GLfloat specular3[] = { 1.0f, 1.0f, 1.0f, 0.3f}; 
GLfloat lightSpot3[] = { 0.3f, 0.3f, 0.3f, 0.3f}; 
GLfloat lightAmbient3[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightDiffuse3[] = { 0.3f, 0.3f, 0.3f, 0.5f };
GLfloat lightPosition3[] = { 2.0f, 2.0f, 10.0f, 1.0f };

GLfloat white[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat red[] = { 1.0f, 0.0f, 0.0f, 1.0f };

GLfloat normalVector[] = { 1.0f, 1.0f, 1.0f };
GLfloat p1[] = { 1.0f, 1.0f, 1.0f };
GLfloat p2[] = { 1.0f, 1.0f, 1.0f };
GLfloat p3[] = { 1.0f, 1.0f, 1.0f };

/* stuff about our window grouped together */
typedef struct {
    Display *dpy;
    int screen;
    Window win;
    GLXContext ctx;
    XSetWindowAttributes attr;
    Bool fs;
    XF86VidModeModeInfo deskMode;
    int x, y;
    unsigned int width, height;
    unsigned int depth;    
} GLWindow;

typedef struct {
    int width;
    int height;
    unsigned char *data;
} textureImage;

/* attributes for a single buffered visual in RGBA format with at least
 * 4 bits per color and a 16 bit depth buffer */
static int attrListSgl[] = {GLX_RGBA, GLX_RED_SIZE, 4, 
    GLX_GREEN_SIZE, 4, 
    GLX_BLUE_SIZE, 4, 
    GLX_DEPTH_SIZE, 16,
    None};

/* attributes for a double buffered visual in RGBA format with at least
 * 4 bits per color and a 16 bit depth buffer */
static int attrListDbl[] = { GLX_RGBA, GLX_DOUBLEBUFFER, 
    GLX_RED_SIZE, 4, 
    GLX_GREEN_SIZE, 4, 
    GLX_BLUE_SIZE, 4, 
    GLX_DEPTH_SIZE, 16,
    None };

GLWindow GLWin;
Bool done;
GLfloat hold;               /* temporary variable */
GLfloat rotX;  /* X Rotation */
GLfloat rotY;  /* Y Rotation */
GLfloat rotZ;  /* Z Rotation */

GLfloat translateZ = 0.0f;
GLfloat translateX = 0.0f;
GLfloat translateY = 0.0f;

GLfloat rotXdiff;  /* X Rotation */
GLfloat rotYdiff;  /* Y Rotation */
GLfloat rotZdiff;  /* Z Rotation */
GLfloat translateZdiff = 0.0f;
GLfloat translateXdiff = 0.0f;
GLfloat translateYdiff = 0.0f;

GLfloat rotXtarget;
GLfloat rotYtarget;
GLfloat rotZtarget;
GLfloat translateZtarget = 0.0f;
GLfloat translateXtarget = 0.0f;
GLfloat translateYtarget = 0.0f;



GLuint  textureFont[512 + 10];  /* Storage For One Texture */
int TextureImageX, TextureImageY;


//+++++++++++++ Loads A Bitmap Image +++++++++++++++++++++++++++++++
unsigned char *LoadBMP(char *Filename){

  FILE *File=NULL;                                // File Handle
  char FilenameLanguage[1000];
  unsigned char *ImageData;
  unsigned short int bfType;
  long int bfOffBits;
  short int biPlanes;
  short int biBitCount;
  long int biSizeImage;
  int i;
  unsigned char temp;

  if (!Filename)                                  // Make Sure A Filename Was Given
  {
    return NULL;                                  // If Not Return NULL
  }


  /* make sure the file is there and open it read-only (binary) */
  if ((File = fopen(Filename, "rb")) == NULL){
    printf("File not found : %s\n", Filename);
    return NULL;
  }
  if(!fread(&bfType, sizeof(short int), 1, File)){
    printf("Error reading : %s\n", Filename);
    return NULL;
  }
  /* check if file is a bitmap */
  if (bfType != 19778){
    printf("No bitmap-format! : %s\n", Filename);
    return NULL;
  }
  /* get the file size */
  /* skip file size and reserved fields of bitmap file header */
  fseek(File, 8, SEEK_CUR);
  /* get the position of the actual bitmap data */
  if (!fread(&bfOffBits, sizeof(long int), 1, File)){
    printf("Error reading OffBits : %s\n", Filename);
    return NULL;
  }
  //printf("Data at Offset: %ld\n", bfOffBits);
  /* skip size of bitmap info header */
  fseek(File, 4, SEEK_CUR);
  /* get the width of the bitmap */
  fread(&TextureImageX, sizeof(int), 1, File);
  //printf("Width of Bitmap: %d\n", texture->width);
  /* get the height of the bitmap */
  fread(&TextureImageY, sizeof(int), 1, File);
  //printf("Height of Bitmap: %d\n", texture->height);
  /* get the number of planes (must be set to 1) */
  fread(&biPlanes, sizeof(short int), 1, File);
  if (biPlanes != 1){
    printf("Wrong BiPlane : %s\n", Filename);
    return NULL;
  }
  /* get the number of bits per pixel */
  if (!fread(&biBitCount, sizeof(short int), 1, File)){
    printf("Wrong BitCount : %s\n", Filename);
    return NULL;
  }
  //printf("Bits per Pixel: %d\n", biBitCount);
  if (biBitCount != 24){
    printf("Wrong depth : %s\n", Filename);
    return NULL;
  }
  /* calculate the size of the image in bytes */
  biSizeImage = TextureImageX * TextureImageY * 3;
  //printf("Size of the image data: %ld\n", biSizeImage);
  if((ImageData = (unsigned char*)malloc(biSizeImage))==NULL){
    printf("Could not allocate memory for %s\n", Filename);
    return NULL;
  }
  /* seek to the actual data */
  fseek(File, bfOffBits, SEEK_SET);
  if (!fread(ImageData, biSizeImage, 1, File)){
    printf("Error reading data : %s\n", Filename);
    return NULL;
  }
  /* swap red and blue (bgr -> rgb) */
  for (i = 0; i < biSizeImage; i += 3){
    temp = ImageData[i];
    ImageData[i] = ImageData[i + 2];
    ImageData[i + 2] = temp;
  }

  fclose(File);

  return ImageData;   // If Load Failed Return 0
}//-------------------------------------------------------------------------- Loads A Bitmap Image -------------------------


//++++++++++++++++++++++++++++++++++++++++ Load Fonts And Convert To Textures +++++++++++++++++++++++++++++++++++++
int LoadFonts(){
  int i, i2, i3;
  int FarbInvers;
  float StartZeit, EndZeit;
  int SchwarzerSchirm=False;
  int AnzeigeEnde=False;
  char Taste;
  char DateiName[1000];
  unsigned char *TextureImageData;


  for(i = 0; i < CHARACTERS; i++){// Loop Through All CHARACTERS Textures
    sprintf(DateiName, "Data/font_%03d.bmp", i);
    if ((TextureImageData = LoadBMP(DateiName))==NULL){
      printf("Could not load %s", DateiName);
      done=True;
      return False;
    }

    printf("Loading %s, TextureImageX=%d, TextureImageY=%d\n", DateiName, TextureImageX, TextureImageX);

    for(i2 = 0; i2 < TextureImageX * TextureImageY * 3; i2++){
      FarbInvers = TextureImageData[i2];
      if(FarbInvers<0){
        printf("FarbInvers=%d", FarbInvers);
      }
      FarbInvers-=255;
      FarbInvers=-FarbInvers;
      TextureImageData[i2]=(char)FarbInvers;
    }

    glGenTextures(1, &textureFont[i]);
    glBindTexture(GL_TEXTURE_2D, textureFont[i]);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImageX, TextureImageY, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImageData);

    //Zugehoerige Maske generieren
    for(i2 = 0; i2 < TextureImageX * TextureImageY * 3; i2++){
      FarbInvers = TextureImageData[i2];
      if(FarbInvers < 0){
        printf("FarbInvers=%d", FarbInvers);
      }
      FarbInvers -= 255;
      FarbInvers = -FarbInvers;
      TextureImageData[i2]=(unsigned char)FarbInvers;
    }

    glGenTextures(1, &textureFont[256 + i]);
    glBindTexture(GL_TEXTURE_2D, textureFont[256 + i]);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImageX, TextureImageY, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImageData);

    if (TextureImageData!=NULL){    // If Texture Image Exists
      free(TextureImageData); // Free The Texture Image Memory
    }

    if(done)return False;
  }//for(i = 0; i < CHARACTERS; i++)


  return True;                                // Return The Status
}//----------------------------------------------------------------- End Load Fonts ------------------------------------------



//+++++++++++++++++++++++++++++++++ 2D text with OpenGL +++++++++++++++++++
void printGLf(GLfloat x, GLfloat y, const char *stringToPrint, int center, int fontColor, GLfloat Size){
GLfloat BreiteF, HoeheF;
unsigned int i, b;


  glPushMatrix();

  BreiteF = Size;
  HoeheF = 0.09f / 0.055f * Size;
  
  glDisable(GL_LIGHTING);
  
  //printf("printGLf: >%s< BreiteF=%lf, HoeheF=%lf\n", stringToPrint, BreiteF, HoeheF);

  if(center == 1){
    x=-1.0f * strlen(stringToPrint) * BreiteF;
  }
  if (strlen(stringToPrint) > 0){
    for(i=0;i<strlen(stringToPrint);i++){
      b=(unsigned int)(stringToPrint[i]);
      //printf("b=%d\n", b);
      //if(string[i]=='Ä')b=196;
      //if(string[i]=='Ö')b=214;
      //if(string[i]=='Ü')b=220;
      //if(string[i]=='ä')b=228;
      //if(string[i]=='ö')b=246;
      //if(string[i]=='ü')b=252;
      //if(string[i]=='ß')b=223;
      //if(string[i]=='´')b=146;
      if(b > 0 && b < 256){
        if(fontColor < 0){//Black Background color if fontcolor < 0
          glColor3f(0.0f, 0.0f, 0.0f);
        }
        else{
          glColor3f(1.0f, 1.0f, 1.0f);
        }
        glEnable(GL_TEXTURE_2D);
        glEnable(GL_BLEND);
        glBlendFunc(GL_DST_COLOR, GL_ZERO);
        glLoadIdentity();
        glTranslatef((float)(x + i * BreiteF * 2.0f), y, -6.0f);
        glBindTexture(GL_TEXTURE_2D, textureFont[256 + b]);
        
        glBegin(GL_QUADS);
            glTexCoord2f(0.0f, 1.0f);
            glVertex3f(-BreiteF, HoeheF, 0.0f);
            glTexCoord2f(1.0f, 1.0f);
            glVertex3f(BreiteF, HoeheF, 0.0f);
            glTexCoord2f(1.0f, 0.0f);
            glVertex3f(BreiteF, -HoeheF, 0.0f);
            glTexCoord2f(0.0f, 0.0f);
            glVertex3f(-BreiteF, -HoeheF, 0.0f);
        glEnd();


        glColor3f(1.0f, 1.0f, 0.0f);//Yellow is the standard
        if(fontColor == 1 || fontColor == -1){
          glColor3f(0.0f, 1.0f, 0.0f);
        }
        if(fontColor == 2 || fontColor == -2){//Orange
          glColor3f(1.0f, 0.75f, 0.0f);
        }
        if(fontColor == 3 || fontColor == -3){//Red
          glColor3f(1.0f, 0.0f, 0.0f);
        }
        if(fontColor == 4 || fontColor == -4){
          glColor3f(0.0f, 1.0f, 1.0f);
        }
        if(fontColor == 5 || fontColor == -5){//Yellow
          glColor3f(1.0f, 1.0f, 0.0f);
        }
        if(fontColor == 6 || fontColor == -6){//Light blue
          glColor3f(0.0f, 0.7f, 1.0f);
        }

        glBlendFunc(GL_ONE, GL_ONE);
        glBindTexture(GL_TEXTURE_2D, textureFont[b]);
        glBegin(GL_QUADS);
            glTexCoord2f(0.0f, 1.0f);
            glVertex3f(-BreiteF, HoeheF, 0.0f);
            glTexCoord2f(1.0f, 1.0f);
            glVertex3f(BreiteF, HoeheF, 0.0f);
            glTexCoord2f(1.0f, 0.0f);
            glVertex3f(BreiteF, -HoeheF, 0.0f);
            glTexCoord2f(0.0f, 0.0f);
            glVertex3f(-BreiteF, -HoeheF, 0.0f);
        glEnd();

      }
    }
    glColor3f(1.0f, 1.0f, 1.0f);
  }
  
  glPopMatrix();


}//----------------------------------------------------------------------------- End Text with OpenGL ---------------------

/* function called when our window is resized (should only happen in window mode) */
void resizeGLScene(unsigned int width, unsigned int height){
    if (height == 0)    /* Prevent A Divide By Zero If The Window Is Too Small */
        height = 1;
    /* Reset The Current Viewport And Perspective Transformation */
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0f, (GLfloat)width / (GLfloat)height, 0.1f, 100.0f);
    //glFrustum (-1.0, 1.0, -1.0, 1.0, 1.5, 20.0);
    //glOrtho(-4.0, 4.0, -4.0, 4.0, 5, 100);
    glMatrixMode(GL_MODELVIEW);
}





//+++++++++++++++++++ Make snapshot and store result in folder PicSequence ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
int ScreenShot(){  
  char fileName[1000];
  char fileName2[1000];
  char picNumberChar[1000];
  char commandLine[1000];

   
  // GLubyte * bits; //RGB bits  
  GLint viewport[4]; //current viewport  
    
  //get current viewport  
  glGetIntegerv(GL_VIEWPORT, viewport);  
  
  uint32_t screenWidth = viewport[2];  
  uint32_t screenHeight = viewport[3];  
  int i, j;
  FILE *saveFile;
  uint32_t BMsize;
  uint32_t tempLong = 0;
  uint16_t tempInt = 0;
  uint32_t BMoffset;
  uint32_t BMheaderSize;
  unsigned char tempChar;
  unsigned char *buffer = (unsigned char *) malloc(screenWidth * screenHeight * 3);

  printf("Starting screenshot.\n", fileName);
   
//  GLubyte bits[3*w*h];
  
  //read pixel from frame buffer  
  glFinish(); //finish all commands of OpenGL  
  glPixelStorei(GL_PACK_ALIGNMENT,1); //or glPixelStorei(GL_PACK_ALIGNMENT,4);  
  glPixelStorei(GL_PACK_ROW_LENGTH, 0);  
  glPixelStorei(GL_PACK_SKIP_ROWS, 0);  
  glPixelStorei(GL_PACK_SKIP_PIXELS, 0);  
    
  //buffer = (GLubyte *)malloc(screenWidth * screenHeight * 3);  
  
  glReadPixels(0, 0, screenWidth, screenHeight, GL_RGB, GL_UNSIGNED_BYTE, buffer);  
  
  printf("Width=%d, height=%d\n", screenWidth, screenHeight);
  
	for ( i = 0; i < screenWidth * screenHeight * 3; i += 3){
		tempChar = buffer[i];
		buffer[i] = buffer[i + 2];
		buffer[i + 2] = tempChar;
	}
  
  strcpy(fileName, "./PicSequence/pic_");
  sprintf(picNumberChar, "%05d.bmp", picNumber);
  strcat(fileName, picNumberChar);

  strcpy(fileName2, "./PicSequence/pic_");
  sprintf(picNumberChar, "%05d.jpg", picNumber);
  strcat(fileName2, picNumberChar);
  
  if ((saveFile = fopen(fileName, "w+b")) == NULL){
      //printf("Highscores not opened for writing: %s\n", filename);
      return 1;
  }
  else{
    printf("%s opened.\n", fileName);
  }
  
  //IplImage * capImg = cvCreateImage( cvSize(w,h), IPL_DEPTH_8U, 3);  
  
  fwrite("BM", 2, 1, saveFile);
  BMsize = screenWidth * screenHeight * 3 + 54;
  fwrite(&BMsize, 4, 1, saveFile);
  tempLong = 0;
  fwrite(&tempLong, 4, 1, saveFile);
  BMoffset = 54;
  fwrite(&BMoffset, 4, 1, saveFile);
  BMheaderSize = 40;
  fwrite(&BMheaderSize, 4, 1, saveFile);
  fwrite(&screenWidth, 4, 1, saveFile);
  fwrite(&screenHeight, 4, 1, saveFile);
  tempInt = 1;
  fwrite(&tempInt, 2, 1, saveFile);
  tempInt = 3 * 8;
  fwrite(&tempInt, 2, 1, saveFile);
  tempLong = 0;
  fwrite(&tempLong, 4, 1, saveFile);
  tempLong = 3 * screenWidth * screenHeight;
  fwrite(&tempLong, 4, 1, saveFile);
  tempLong = 0;
  fwrite(&tempLong, 4, 1, saveFile);
  tempLong = 0;
  fwrite(&tempLong, 4, 1, saveFile);
  tempLong = 0;
  fwrite(&tempLong, 4, 1, saveFile);
  tempLong = 0;
  fwrite(&tempInt, 4, 1, saveFile);
  
  printf("Writing buffer to file.\n");
  
  if(!fwrite(buffer, 3 * screenWidth * screenHeight, 1, saveFile)){
    printf("Error writing file ImageData.bmp!\n");
    return 1;
  }    
  
  fclose(saveFile);
  
  free(buffer);
  
  printf("%s saved.\n", fileName);
  
  strcpy(commandLine, "convert ");
  strcat(commandLine, fileName);
  strcat(commandLine, " -quality 90 ");
  strcat(commandLine, fileName2);
  printf("executing %s...\n", commandLine);
  system(commandLine);
  strcpy(commandLine, "rm ");
  strcat(commandLine, fileName);
  system(commandLine);
  printf("...done.\n");

  picNumber++;
  
  return 0;
   
}  


/* general OpenGL initialization function */
int initGL(GLvoid){
    int x, y;
    float chargeX = gridX / 2.0f, chargeY = gridY / 2.0f;
    float totalDistance = 0.0f, totalForce = 0.0f;

    printf("Loading fonts...\n");
    if (!LoadFonts())                // Jump To Texture Loading Routine
    {
      return False;                        // If Texture Didn't Load Return False
    }
    else{
      printf("Fonts OK.\n");
    }


    glEnable(GL_TEXTURE_2D);       /* Enable Texture Mapping */
    glShadeModel(GL_SMOOTH);
    
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glClearDepth(1.0f);
    glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
    glEnable(GL_LINE_SMOOTH);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glShadeModel(GL_SMOOTH);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    glPolygonMode(GL_BACK, GL_LINE);
    glPolygonMode(GL_FRONT, GL_LINE);



    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, white);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 80.0f);

    glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient1);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse1);
    glLightfv(GL_LIGHT0, GL_POSITION, lightPosition0);


    glLightfv(GL_LIGHT1, GL_AMBIENT, lightAmbient1);
    glLightfv(GL_LIGHT1, GL_DIFFUSE, lightDiffuse1);
    glLightfv(GL_LIGHT1, GL_POSITION, lightPosition1);
    glLightfv(GL_LIGHT1,GL_SPECULAR,specular1);
    glEnable(GL_LIGHT1);

    glLightfv(GL_LIGHT2, GL_DIFFUSE, lightDiffuse2);
    
    glLightfv(GL_LIGHT3, GL_DIFFUSE, lightDiffuse3);
    
    glEnable(GL_LIGHTING);
    
    
    /* we use resizeGLScene once to set up our initial perspective */
    resizeGLScene(GLWin.width, GLWin.height);
    glFlush();
    return True;
}


void calculateNormalVector(){
  GLfloat length;
  
  GLfloat p2minp1[3];
  GLfloat p3minp1[3];
 
  p2minp1[0] = p2[0] - p1[0];
  p2minp1[1] = p2[1] - p1[1];
  p2minp1[2] = p2[2] - p1[2];
  
  p3minp1[0] = -(p3[0] - p1[0]);
  p3minp1[1] = -(p3[1] - p1[1]);
  p3minp1[2] = -(p3[2] - p1[2]);
  
  
  //Normalize Z-component to 1.0f for maximum reflection
  
  normalVector[2] = p2minp1[0] * p3minp1[1] - p2minp1[1] * p3minp1[0];
  normalVector[0] = (p2minp1[1] * p3minp1[2] - p2minp1[2] * p3minp1[1]);
  normalVector[1] = (p2minp1[2] * p3minp1[0] - p2minp1[0] * p3minp1[2]);
  
  length = sqrt(pow(normalVector[0], 2.0) + pow(normalVector[1], 2.0) + pow(normalVector[2], 2.0));

  normalVector[0] = normalVector[0] / length;
  normalVector[1] = normalVector[1] / length;
  normalVector[2] = normalVector[2] / length;
  
  //normalVector[2] = 1.0;
  
//  if(normalVector[2] < 0.0f){
//    normalVector[2] = -normalVector[2];
//  }
  
}


//++++++++++++++++++++++++++++++++++ Draw sphere ++++++++++++++++++++++++++++++++++++++++++++++++++
void drawSphere(GLfloat x1, GLfloat y1, GLfloat z1, double radius, int nSlice, int nStack, int enableTranslucentSphere){
    int i, j;
    double phi; //
    double theta; //long
    float sp[129][129][3]; //Vertex
    float *sp1,*sp2,*sp3,*sp4;
    float spn[129][129][3]; //Vertex
    float *spn1,*spn2,*spn3,*spn4;




  glPushMatrix();
  glTranslatef( x1,y1,z1 );  

  if(enableTranslucentSphere == 1){

    glBlendFunc(GL_SRC_ALPHA,GL_ONE);
    
    glEnable(GL_BLEND);
    glDisable(GL_DEPTH_TEST);
    glColor4f(0.2f,0.2f,0.2f,0.1f); 

  }

  

    if(nSlice > 128) nSlice = 128;
    if(nStack > 128) nStack = 128;

    //Vertex
    for(i = 0;i <= nSlice;i++){   
        phi = 2.0 * M_PI * (double)i / (double)nSlice;
        for(j = 0;j <= nStack;j++){   
            theta = M_PI * (double)j / (double)nStack;
            sp[i][j][0] = (float)(radius * sin(theta) * cos(phi));//x
            sp[i][j][1] = (float)(radius * sin(theta) * sin(phi));//y
            sp[i][j][2] = (float)(radius * cos(theta));           //z
        }
    }

    //Normals (radius = 1);
    for(i = 0;i <= nSlice;i++){   
        phi = 2.0 * M_PI * (double)i / (double)nSlice;
        for(j = 0;j <= nStack;j++){   
            theta = M_PI * (double)j / (double)nStack;
            spn[i][j][0] = (float)(1.0 * sin(theta) * cos(phi));//x
            spn[i][j][1] = (float)(1.0 * sin(theta) * sin(phi));//y
            spn[i][j][2] = (float)(1.0 * cos(theta));           //z
        }
    }


    //Top(j=0)
    for(i = 0;i < nSlice; i++){
        sp1 = sp[i][0];
        sp2 = sp[i][1];
        sp3 = sp[i+1][1]; 

        spn1 = spn[i][0];
        spn2 = spn[i][1];
        spn3 = spn[i+1][1]; 

        glBegin(GL_TRIANGLES);            
            glNormal3fv(spn1); glVertex3fv(sp1);
            glNormal3fv(spn2); glVertex3fv(sp2);
            glNormal3fv(spn3); glVertex3fv(sp3);
        glEnd();
    }
    //Bottom
    j=nStack-1;
    for(i = 0;i < nSlice; i++){
        sp1 = sp[i][j];
        sp2 = sp[i][j+1];
        sp3 = sp[i+1][j]; 

        spn1 = spn[i][j];
        spn2 = spn[i][j+1];
        spn3 = spn[i+1][j]; 

        glBegin(GL_TRIANGLES);
            glNormal3fv(spn1); glVertex3fv(sp1);
            glNormal3fv(spn2); glVertex3fv(sp2);
            glNormal3fv(spn3); glVertex3fv(sp3);
        glEnd();
    }

    for(i = 0;i < nSlice;i++){
        for(j = 1;j < nStack-1; j++){
            sp1 = sp[i][j];
            sp2 = sp[i][j+1];
            sp3 = sp[i+1][j+1];
            sp4 = sp[i+1][j];

            spn1 = spn[i][j];
            spn2 = spn[i][j+1];
            spn3 = spn[i+1][j+1];
            spn4 = spn[i+1][j];

            glBegin(GL_QUADS);
                glNormal3fv(spn1); glVertex3fv(sp1);
                glNormal3fv(spn2); glVertex3fv(sp2);
                glNormal3fv(spn3); glVertex3fv(sp3);
                glNormal3fv(spn4); glVertex3fv(sp4);
            glEnd();
        }
    }
    
    glPopMatrix();

    glPolygonMode(GL_BACK, GL_FILL);
    glPolygonMode(GL_FRONT, GL_FILL);

    glDisable(GL_BLEND);
    glEnable(GL_DEPTH_TEST);
    glColor4f(1.0f,1.0f,1.0f,1.0f); 


}

//++++++++++++++++++++++++++++++++++ Draw cylinder ++++++++++++++++++++++++++++++++++++++++++++++++++
void drawCylinder(float x1, float y1, float z1, float x2,float y2, float z2, float radius1, float radius2, int subdivisions){


  GLfloat vx = x2-x1;
  GLfloat vy = y2-y1;
  GLfloat vz = z2-z1;

  //handle the degenerate case with an approximation

  if(vz < 0.001 && vz > 0.0){
    vz = 0.001;
  }
  if(vz > -0.001 && vz < 0.0){
    vz = -0.001;
  }
  if(vz == 0.0){
    vz = 0.001;
  }
  
  
  GLfloat v = sqrt( vx*vx + vy*vy + vz*vz );
//  if(v<0.001)v=0.001;
  GLfloat ax = 57.2957795*acos( vz * 1000.0 / (v * 1000.0f) );//multiply with 1000.0f to avoid wrong result at small angles
  if ( vz < 0.0 ){
    ax = -ax;
  }
  GLfloat rx = -vy*vz * 1000.0f;//multiply with 1000.0f to avoid wrong result at small angles
  GLfloat ry = vx*vz * 1000.0f;//multiply with 1000.0f to avoid wrong result at small angles

  GLUquadricObj *quadric=gluNewQuadric();
  gluQuadricNormals(quadric, GLU_SMOOTH);


  glPushMatrix();
  glTranslatef( x1,y1,z1 );
  glRotatef(ax, rx, ry, 0.000001);
  
  //printf("vx = %lf, vy = %lf, vz = %lf, ax = %lf, rx = %lf, ry = %lf, v=%lf\n", vx, vy, vz, ax, rx, ry, v);
  
  //draw the cylinder
  gluCylinder(quadric, radius1, radius2, v, 32, 1);
  gluQuadricOrientation(quadric,GLU_INSIDE);
  //draw the first cap
  gluDisk( quadric, 0.0, radius1, 32, 1);
  glTranslatef( 0,0,v );
  //draw the second cap
  gluQuadricOrientation(quadric,GLU_OUTSIDE);
  gluDisk( quadric, 0.0, radius2, 32, 1);
  

  glPopMatrix();

  gluDeleteQuadric(quadric);
}




void paintAxes(GLfloat xMax, GLfloat yMax, GLfloat zMax, int orientation){
    GLfloat peak, peakLength = 0.5f;

  
    glDisable(GL_LIGHTING);
    glDisable(GL_BLEND);
    glEnable(GL_DEPTH_TEST);
    
    
    glLineWidth(5.5);
    
    //X-axis
    glColor3f(axeXColor[0], axeXColor[1], axeXColor[2]);

    peak = xMax + peakLength * 1.1f;
    glBegin(GL_LINES);
    glVertex3f(0.0, 0.0, 0.0);
    glVertex3f(peak, 0.0f, 0.0f);
    glEnd();
    glBegin(GL_LINES);
    glVertex3f(peak - peakLength, peakLength, 0.0f);
    glVertex3f(peak, 0.0f, 0.0f);
    glEnd();
    glBegin(GL_LINES);
    glVertex3f(peak - peakLength, -peakLength, 0.0f);
    glVertex3f(peak, 0.0f, 0.0f);
    glEnd();

    //Paint X
    glBegin(GL_LINES);
    glVertex3f(peak + peakLength * 0.5, -peakLength/2.0f, 0.0f);
    glVertex3f(peak + peakLength * 1.5, peakLength/2.0f, 0.0f);
    glEnd();
    glBegin(GL_LINES);
    glVertex3f(peak + peakLength * 1.5, -peakLength/2.0f, 0.0f);
    glVertex3f(peak + peakLength * 0.5, peakLength/2.0f, 0.0f);
    glEnd();


    //Z-axis
    peak = zMax + peakLength * 1.1f;
    if(orientation == 1 || orientation == 2){
      glColor3f(axeYColor[0], axeYColor[1], axeYColor[2]);
    }
    else{
      glColor3f(axeZColor[0], axeZColor[1], axeZColor[2]);
    }
    
    glBegin(GL_LINES);
    glVertex3f(0.0, 0.0, 0.0);
    glVertex3f(0.0f, 0.0f, -peak);
    glEnd();
    if(zMax > 0.0f){
      glBegin(GL_LINES);
      glVertex3f(peakLength, 0.0f, -(peak - peakLength));
      glVertex3f(0.0f, 0.0f, -peak);
      glEnd();
      glBegin(GL_LINES);
      glVertex3f(-peakLength, 0.0f, -(peak - peakLength));
      glVertex3f(0.0f, 0.0f, -peak);
      glEnd();
    }
    else{
      glBegin(GL_LINES);
      glVertex3f(peakLength, 0.0f, -peak - peakLength);
      glVertex3f(0.0f, 0.0f, -peak);
      glEnd();
      glBegin(GL_LINES);
      glVertex3f(-peakLength, 0.0f, -peak - peakLength);
      glVertex3f(0.0f, 0.0f, -peak);
      glEnd();
    }
    if(orientation == 1 || orientation == 2){
      //Paint Y at Z-axis when orientation is swapped
      glBegin(GL_LINES);
      glVertex3f( -peakLength/2.0f, 0.0f, -(peak + peakLength * 1.5));
      glVertex3f( 0.0f, 0.0f, -(peak + peakLength * 0.8));
      glEnd();
      glBegin(GL_LINES);
      glVertex3f( peakLength/2.0f, 0.0f, -(peak + peakLength * 1.5));
      glVertex3f( 0.0f, 0.0f, -(peak + peakLength * 0.8));
      glEnd();
      glBegin(GL_LINES);
      glVertex3f( 0.0f, 0.0f, -(peak + peakLength * 0.8));
      glVertex3f( 0.0f, 0.0f, -(peak + peakLength * 0.5));
      glEnd();
    }
    else{
      //Paint Z
      if(zMax > 0.0f){
        glBegin(GL_LINES);
        glVertex3f( -peakLength/2.0f, 0.0f, -(peak + peakLength * 1.5));
        glVertex3f( peakLength/2.0f, 0.0f, -(peak + peakLength * 1.5));
        glEnd();
        if(orientation == 3){
          glBegin(GL_LINES);
          glVertex3f( -peakLength/2.0f, 0.0f, -(peak + peakLength * 1.5));
          glVertex3f( peakLength/2.0f, 0.0f, -(peak + peakLength * 0.5));
          glEnd();
        }
        else{
          glBegin(GL_LINES);
          glVertex3f( peakLength/2.0f, 0.0f, -(peak + peakLength * 1.5));
          glVertex3f( -peakLength/2.0f, 0.0f, -(peak + peakLength * 0.5));
          glEnd();
        }
        glBegin(GL_LINES);
        glVertex3f( -peakLength/2.0f, 0.0f, -(peak + peakLength * 0.5));
        glVertex3f( peakLength/2.0f, 0.0f, -(peak + peakLength * 0.5));
        glEnd();
      }
      else{
        glBegin(GL_LINES);
        glVertex3f( -peakLength/2.0f, 0.0f, -peak + peakLength * 1.5);
        glVertex3f( peakLength/2.0f, 0.0f, -peak + peakLength * 1.5);
        glEnd();
        if(orientation == 3){
          glBegin(GL_LINES);
          glVertex3f( peakLength/2.0f, 0.0f, -peak + peakLength * 1.5);
          glVertex3f( -peakLength/2.0f, 0.0f, -peak + peakLength * 0.5);
          glEnd();
        }
        else{
          glBegin(GL_LINES);
          glVertex3f( -peakLength/2.0f, 0.0f, -peak + peakLength * 1.5);
          glVertex3f( peakLength/2.0f, 0.0f, -peak + peakLength * 0.5);
          glEnd();
        }
        glBegin(GL_LINES);
        glVertex3f( -peakLength/2.0f, 0.0f, -peak + peakLength * 0.5);
        glVertex3f( peakLength/2.0f, 0.0f, -peak + peakLength * 0.5);
        glEnd();
      }
    }


    if(axeYColor[0] > 0.0f || axeYColor[1] > 0.0f || axeYColor[2] > 0.0f){//Dont paint if color is black
      //Paint Y-axis
      peak = yMax + peakLength * 1.1f;
      if(orientation == 1 || orientation == 2){
        glColor3f(axeZColor[0], axeZColor[1], axeZColor[2]);
      }
      else{
        glColor3f(axeYColor[0], axeYColor[1], axeYColor[2]);
      }
      

      glBegin(GL_LINES);
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(0.0f, peak, 0.0f);
      glEnd();
      glBegin(GL_LINES);
      glVertex3f(peakLength, peak - peakLength, 0.0f);
      glVertex3f(0.0f, peak, 0.0f);
      glEnd();
      glBegin(GL_LINES);
      glVertex3f(-peakLength, peak - peakLength, 0.0f);
      glVertex3f(0.0f, peak, 0.0f);
      glEnd();
      
      if(orientation == 1 || orientation == 2){
        //Paint Z at Y-axis when orientation is swapped
        if(zMax > 0.0f){
          glBegin(GL_LINES);
          glVertex3f( -peakLength/2.0f, (peak + peakLength * 1.5), 0.0f);
          glVertex3f( peakLength/2.0f, (peak + peakLength * 1.5), 0.0f);
          glEnd();
          if(orientation == 1){
            glBegin(GL_LINES);
            glVertex3f( peakLength/2.0f, (peak + peakLength * 1.5), 0.0f);
            glVertex3f( -peakLength/2.0f, (peak + peakLength * 0.5), 0.0f);
            glEnd();
          }
          else{
            glBegin(GL_LINES);
            glVertex3f( -peakLength/2.0f, (peak + peakLength * 1.5), 0.0f);
            glVertex3f( peakLength/2.0f, (peak + peakLength * 0.5), 0.0f);
            glEnd();
          }
          glBegin(GL_LINES);
          glVertex3f( -peakLength/2.0f, (peak + peakLength * 0.5), 0.0f);
          glVertex3f( peakLength/2.0f, (peak + peakLength * 0.5), 0.0f);
          glEnd();
        }
        else{
          glBegin(GL_LINES);
          glVertex3f( -peakLength/2.0f, peak + peakLength * 1.5, 0.0f);
          glVertex3f( peakLength/2.0f, peak + peakLength * 1.5, 0.0f);
          glEnd();
          glBegin(GL_LINES);
          glVertex3f( peakLength/2.0f, peak + peakLength * 1.5, 0.0f);
          glVertex3f( -peakLength/2.0f, peak + peakLength * 0.5, 0.0f);
          glEnd();
          glBegin(GL_LINES);
          glVertex3f( -peakLength/2.0f, peak + peakLength * 0.5, 0.0f);
          glVertex3f( peakLength/2.0f, peak + peakLength * 0.5, 0.0f);
          glEnd();
        }
      }
      else{
        //Paint Y
        glBegin(GL_LINES);
        glVertex3f( -peakLength/2.0f, peak + peakLength * 1.5, 0.0f);
        glVertex3f( 0.0f, peak + peakLength * 0.8, 0.0f);
        glEnd();
        glBegin(GL_LINES);
        glVertex3f( peakLength/2.0f, peak + peakLength * 1.5, 0.0f);
        glVertex3f( 0.0f, peak + peakLength * 0.8, 0.0f);
        glEnd();
        glBegin(GL_LINES);
        glVertex3f( 0.0f, peak + peakLength * 0.8, 0.0f);
        glVertex3f( 0.0f, peak + peakLength * 0.5, 0.0f);
        glEnd();
      }
    }
    

}

/* Here goes our drawing code */
int drawGLScene(GLvoid){
    int x, z, i, i2;
    GLfloat x1, x2, x3, x4, y1, y2, y3, y4, z1, z2, z3, z4;
    GLfloat chargeX = 0.0f, chargeY = 0.0f;
    GLfloat totalDistance;
    GLfloat totalForce, potential;
    GLfloat steps = 200.0f;
    char textLine[1000];
    float float_x, float_z, float_xb, float_zb;
    GLfloat k = 1.0 / (4.0 * PI * 8.85419E-12);
    GLfloat forceX, forceY, forceZ;
    double fieldLineStep = 0.1;
    GLfloat arrowRadius = 3.0;
    GLfloat peak = 0.0;
    
    usleep(slowDownMS);
    
    yMax = xMax / 16.0f * 9.0f;
    coordScale = 2.0 * xMax / gridX;
    
        
    
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
    glLightfv(GL_LIGHT1, GL_POSITION, lightPosition1);
    
    glLoadIdentity();
    glTranslatef(translateX, translateY, translateZ);
    glRotatef(rotX, 1.0f, 0.0f, 0.0f);  /* rotate on the X axis */
    glRotatef(rotY, 0.0f, 1.0f, 0.0f);  /* rotate on the Y axis */
    glRotatef(rotZ, 0.0f, 0.0f, 1.0f);  /* rotate on the Z axis */
    
    if(animation >= 200 && animation < 300){
      paintAxes(5.5, 4.0, -4.0, 0);
    }


    glEnable(GL_LIGHTING);
    

    if(enableParticles == 1){
      //Draw particles
      glEnable(GL_LIGHTING);
      glPolygonMode(GL_BACK, GL_FILL);
      glPolygonMode(GL_FRONT, GL_FILL);
      for(i=0; i< particleNumber; i++){
        enableTransLucentSphere = 0;
        if(particles[i][4] < 0.0f){
          glLightfv(GL_LIGHT1, GL_AMBIENT, lightBlue3);
          glLightfv(GL_LIGHT1, GL_DIFFUSE, lightBlue3);
          glLightfv(GL_LIGHT1, GL_SPECULAR, lightBlue3);
        }
        else{
          glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed1);
          glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed1);
          glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);
        }
        if(particles[i][5] == 1.0f){//"Neutral" particle
          glLightfv(GL_LIGHT1, GL_AMBIENT, lightWhite1);
          glLightfv(GL_LIGHT1, GL_DIFFUSE, lightWhite1);
          glLightfv(GL_LIGHT1, GL_SPECULAR, lightWhite1);
        }
        if(particles[i][5] == 2.0f){//Almost "neutral" particle
          glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed3);
          glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed3);
          glLightfv(GL_LIGHT1, GL_SPECULAR, lightRed3);
        }
        if(particles[i][5] == 3.0f){//Translucent blue particle
          glLightfv(GL_LIGHT1, GL_AMBIENT, lightBlue3);
          glLightfv(GL_LIGHT1, GL_DIFFUSE, lightBlue3);
          glLightfv(GL_LIGHT1, GL_SPECULAR, lightBlue3);
          enableTransLucentSphere = 1;
        }
        if(particles[i][5] != -1.0f){//Invisible particle
          drawSphere(particles[i][0],particles[i][1], particles[i][2], particles[i][3], 40, 40, enableTransLucentSphere);
        }
      }
    }
    
    
        

    if(firstStart == 0 && animationPause == 0){
      translateX += translateXdiff;
      translateY += translateYdiff;
      translateZ += translateZdiff;
      rotX += rotXdiff;
      rotY += rotYdiff;
      rotZ += rotZdiff;
    }

//####################################################################################
//############# Paint field lines of a positive and a negative charge ################
//####################################################################################

      if(animation == 200 && firstStart < 2){
        GLfloat fieldLineCoords[10000][4];
        GLfloat fieldLineLength = 40.0;
        double alpha, beta;
        
        glEnable(GL_LIGHTING);
        glPolygonMode(GL_BACK, GL_FILL);
        glPolygonMode(GL_FRONT, GL_FILL);


        glLightfv(GL_LIGHT1, GL_AMBIENT, lightGreen1);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, lightGreen1);
        glLightfv(GL_LIGHT1, GL_SPECULAR, lightGreen1);


        for(i = 0; i < 8; i++){
          alpha = i * 2.0 * 3.1415927 / 8;
          fieldLineCoords[i][0] = particles[0][0] + sin(alpha) * particles[0][3];
          fieldLineCoords[i][1] = particles[0][1] + cos(alpha) * particles[0][3];
          fieldLineCoords[i][2] = 0.0;
        }
        for(i = 8; i < 12; i++){
          alpha = i * 2.0 * 3.1415927 / 4;
          fieldLineCoords[i][0] = particles[0][0] + sin(alpha) * particles[0][3] * 0.5;
          fieldLineCoords[i][1] = particles[0][1] + cos(alpha) * particles[0][3] * 0.5;
          fieldLineCoords[i][2] = particles[0][3] * 0.5;
        }
        for(i = 12; i < 16; i++){
          alpha = i * 2.0 * 3.1415927 / 4;
          fieldLineCoords[i][0] = particles[0][0] + sin(alpha) * particles[0][3] * 0.5;
          fieldLineCoords[i][1] = particles[0][1] + cos(alpha) * particles[0][3] * 0.5;
          fieldLineCoords[i][2] = -particles[0][3] * 0.5;
        }

        fieldLineCoords[16][0] = particles[0][0];
        fieldLineCoords[16][1] = particles[0][1];
        fieldLineCoords[16][2] = -particles[0][3];

        fieldLineCoords[17][0] = particles[0][0];
        fieldLineCoords[17][1] = particles[0][1];
        fieldLineCoords[17][2] = particles[0][3];

        fieldLineCoords[18][0] = 40.0;
        fieldLineCoords[18][1] = 0.0;
        fieldLineCoords[18][2] = 0.0;
        
        for(i=0; i<18; i++){
          fieldLineCoords[i][3] = 0.0;
          if(fieldLineCoords[i][0] > particles[0][0]){
            fieldLineCoords[i][3] = 3.0;
          }
        }
        fieldLineCoords[18][3] = 1.0;

        for(i2 = 0; i2 < 19; i2++){
          for(i = 0; i < fieldLineEndUp; i++){
            totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
            totalForce = k * particles[0][4] / pow(totalDistance, 2.0);
            forceX = (fieldLineCoords[i2][0] - particles[0][0]) / totalDistance * totalForce;
            forceY = (fieldLineCoords[i2][1] - particles[0][1]) / totalDistance * totalForce;
            forceZ = (fieldLineCoords[i2][2] - particles[0][2]) / totalDistance * totalForce;

            totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
            totalForce = k * particles[1][4] / pow(totalDistance, 2.0);
            forceX += (fieldLineCoords[i2][0] - particles[1][0]) / totalDistance * totalForce;
            forceY += (fieldLineCoords[i2][1] - particles[1][1]) / totalDistance * totalForce;
            forceZ += (fieldLineCoords[i2][2] - particles[1][2]) / totalDistance * totalForce;

            totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0) + pow(forceZ, 2.0));
            x2 = fieldLineCoords[i2][0] + forceX / totalForce * fieldLineStep;
            y2 = fieldLineCoords[i2][1] + forceY / totalForce * fieldLineStep;
            z2 = fieldLineCoords[i2][2] + forceZ / totalForce * fieldLineStep;
            
            drawCylinder(fieldLineCoords[i2][0], fieldLineCoords[i2][1], fieldLineCoords[i2][2], x2, y2, z2, 0.03, 0.03, 32);
            
            if(fieldLineCoords[i2][3] == 3.0){
              if(x2 > 0.0){
                drawCylinder(x1, y1, z1, x2, y2, z2, 0.1, 0.03, 32);
                fieldLineCoords[i2][3] = 2.0;
              }
            }
            else{
              if(fieldLineCoords[i2][3] == 0.0){
                totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
                if(totalDistance > arrowRadius){
                  drawCylinder(x1, y1, z1, x2, y2, z2, 0.1, 0.03, 32);
                  fieldLineCoords[i2][3] = 1.0;
                }
              }
              if(fieldLineCoords[i2][3] == 1.0){
                totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
                if(totalDistance < arrowRadius){
                  drawCylinder(x1, y1, z1, x2, y2, z2, 0.1, 0.03, 32);
                  fieldLineCoords[i2][3] = 2.0;
                }
              }
            }

            x1 = fieldLineCoords[i2][0];
            y1 = fieldLineCoords[i2][1];
            z1 = fieldLineCoords[i2][2];

            fieldLineCoords[i2][0] = x2;
            fieldLineCoords[i2][1] = y2;
            fieldLineCoords[i2][2] = z2;
            
            totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
            if(totalDistance < particles[1][3]){//Field line reached negative charge
              break;
            }
          }//for(i = 0; i < fieldLineEndUp; i++){
        }//for(i2 = 0; i2 < 2; i2++){
        
        glColor3f(1.0f, 0.0f, 0.0f);

        glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed1);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed1);
        glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);
        drawSphere(particles[0][0], particles[0][1], particles[0][2], particles[0][3], 40.0, 40.0, 0);      

        glLightfv(GL_LIGHT1, GL_AMBIENT, lightBlue1);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, lightBlue1);
        glLightfv(GL_LIGHT1, GL_SPECULAR, lightBlue1);
        drawSphere(particles[1][0], particles[1][1], particles[1][2], particles[1][3], 40.0, 40.0, 0);      

        glLightfv(GL_LIGHT1, GL_AMBIENT, lightAmbient1);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, lightDiffuse1);
        glLightfv(GL_LIGHT1,GL_SPECULAR,specular1);
        
        if(animationPause == 0){
          if(fieldLineEndUp < 1000){
            fieldLineEndUp++;
          }
          if(fieldLineEndUp > 40){
            rotXdiff = 0.5;
            if(translateZ > -27.0){
              translateZdiff = -0.05;
            }
            else{
              translateZdiff = 0.0;
            }
          }
          if(rotX > 370.0){
            savePics = 0;
            printf("...un ferddisch!\n");
            done = True;         // Exit programm !!!
          }
        }
      }



//###########################################################################################
//############### Print text messages #######################################################
//###########################################################################################

    glEnable(GL_TEXTURE_2D);       /* Enable Texture Mapping */
    glEnable(GL_BLEND);
    glDisable(GL_LIGHTING);
    glEnable(GL_DEPTH_TEST);
    //glDisable(GL_BLEND);
    glLoadIdentity();
    glColor3f(1.0f, 1.0f, 0.0f);
    if(animation >=200 && animation < 300){
      sprintf(textLine, "3D field line pattern");
      printGLf(-4.3, 2.3, textLine, 0, 1, 0.05f);
    }
        
    glDisable(GL_TEXTURE_2D);       /* Enable Texture Mapping */
    glDisable(GL_BLEND);
    
    
    
    glFlush();
    glXSwapBuffers(GLWin.dpy, GLWin.win);

    if(firstStart > 0){
      firstStart--;
    }
    else{
      if(savePics == 1){
        if(picNumber >= startPicNumber){
          ScreenShot();
        }
        else{
          picNumber++;
        }
      }
    }

    return True;    
}

/* function to release/destroy our resources and restoring the old desktop */
GLvoid killGLWindow(GLvoid){
  if (GLWin.ctx){
    if (!glXMakeCurrent(GLWin.dpy, None, NULL)){
      printf("Could not release drawing context.\n");
    }
    glXDestroyContext(GLWin.dpy, GLWin.ctx);
    GLWin.ctx = NULL;
  }
  /* switch back to original desktop resolution if we were in fs */
  if (GLWin.fs){
    XF86VidModeSwitchToMode(GLWin.dpy, GLWin.screen, &GLWin.deskMode);
    XF86VidModeSetViewPort(GLWin.dpy, GLWin.screen, 0, 0);
  }
  XCloseDisplay(GLWin.dpy);
}

/* this function creates our window and sets it up properly */
/* FIXME: bits is currently unused */
Bool createGLWindow(char* title, int width, int height, int bits, Bool fullscreenflag){
    XVisualInfo *vi;
    Colormap cmap;
    int dpyWidth, dpyHeight;
    int i;
    int glxMajorVersion, glxMinorVersion;
    int vidModeMajorVersion, vidModeMinorVersion;
    XF86VidModeModeInfo **modes;
    int modeNum;
    int bestMode;
    Atom wmDelete;
    Window winDummy;
    unsigned int borderDummy;
    
    GLWin.fs = fullscreenflag;
    /* set best mode to current */
    bestMode = 0;
    /* get a connection */
    GLWin.dpy = XOpenDisplay(0);
    GLWin.screen = DefaultScreen(GLWin.dpy);
    XF86VidModeQueryVersion(GLWin.dpy, &vidModeMajorVersion,
        &vidModeMinorVersion);
    printf("XF86VidModeExtension-Version %d.%d\n", vidModeMajorVersion,
        vidModeMinorVersion);
    XF86VidModeGetAllModeLines(GLWin.dpy, GLWin.screen, &modeNum, &modes);
    /* save desktop-resolution before switching modes */
    GLWin.deskMode = *modes[0];
    /* look for mode with requested resolution */
    for (i = 0; i < modeNum; i++)
    {
        if ((modes[i]->hdisplay == width) && (modes[i]->vdisplay == height))
        {
            bestMode = i;
        }
    }
    /* get an appropriate visual */
    vi = glXChooseVisual(GLWin.dpy, GLWin.screen, attrListDbl);
    if (vi == NULL)
    {
        vi = glXChooseVisual(GLWin.dpy, GLWin.screen, attrListSgl);
        printf("Only Singlebuffered Visual!\n");
    }
    else
    {
        printf("Got Doublebuffered Visual!\n");
    }
    glXQueryVersion(GLWin.dpy, &glxMajorVersion, &glxMinorVersion);
    printf("glX-Version %d.%d\n", glxMajorVersion, glxMinorVersion);
    /* create a GLX context */
    GLWin.ctx = glXCreateContext(GLWin.dpy, vi, 0, GL_TRUE);
    /* create a color map */
    cmap = XCreateColormap(GLWin.dpy, RootWindow(GLWin.dpy, vi->screen),
        vi->visual, AllocNone);
    GLWin.attr.colormap = cmap;
    GLWin.attr.border_pixel = 0;

    if (GLWin.fs)
    {
        XF86VidModeSwitchToMode(GLWin.dpy, GLWin.screen, modes[bestMode]);
        XF86VidModeSetViewPort(GLWin.dpy, GLWin.screen, 0, 0);
        dpyWidth = modes[bestMode]->hdisplay;
        dpyHeight = modes[bestMode]->vdisplay;
        printf("Resolution %dx%d\n", dpyWidth, dpyHeight);
        XFree(modes);
    
        /* create a fullscreen window */
        GLWin.attr.override_redirect = True;
        GLWin.attr.event_mask = ExposureMask | KeyPressMask | ButtonPressMask |
            StructureNotifyMask;
        GLWin.win = XCreateWindow(GLWin.dpy, RootWindow(GLWin.dpy, vi->screen),
            0, 0, dpyWidth, dpyHeight, 0, vi->depth, InputOutput, vi->visual,
            CWBorderPixel | CWColormap | CWEventMask | CWOverrideRedirect,
            &GLWin.attr);
        XWarpPointer(GLWin.dpy, None, GLWin.win, 0, 0, 0, 0, 0, 0);
		XMapRaised(GLWin.dpy, GLWin.win);
        XGrabKeyboard(GLWin.dpy, GLWin.win, True, GrabModeAsync,
            GrabModeAsync, CurrentTime);
        XGrabPointer(GLWin.dpy, GLWin.win, True, ButtonPressMask,
            GrabModeAsync, GrabModeAsync, GLWin.win, None, CurrentTime);
    }
    else
    {
        /* create a window in window mode*/
        GLWin.attr.event_mask = ExposureMask | KeyPressMask | ButtonPressMask |
            StructureNotifyMask;
        GLWin.win = XCreateWindow(GLWin.dpy, RootWindow(GLWin.dpy, vi->screen),
            0, 0, width, height, 0, vi->depth, InputOutput, vi->visual,
            CWBorderPixel | CWColormap | CWEventMask, &GLWin.attr);
        /* only set window title and handle wm_delete_events if in windowed mode */
        wmDelete = XInternAtom(GLWin.dpy, "WM_DELETE_WINDOW", True);
        XSetWMProtocols(GLWin.dpy, GLWin.win, &wmDelete, 1);
        XSetStandardProperties(GLWin.dpy, GLWin.win, title,
            title, None, NULL, 0, NULL);
        XMapRaised(GLWin.dpy, GLWin.win);
    }       
    /* connect the glx-context to the window */
    glXMakeCurrent(GLWin.dpy, GLWin.win, GLWin.ctx);
    XGetGeometry(GLWin.dpy, GLWin.win, &winDummy, &GLWin.x, &GLWin.y,
        &GLWin.width, &GLWin.height, &borderDummy, &GLWin.depth);
    printf("Depth %d\n", GLWin.depth);
    if (glXIsDirect(GLWin.dpy, GLWin.ctx)) 
        printf("Congrats, you have Direct Rendering!\n");
    else
        printf("Sorry, no Direct Rendering possible!\n");
    if (!initGL())
    {
        printf("Could not initialize OpenGL.\nAborting...\n");
        return False;
    }        
    return True;    
}
void keyPressed(KeySym key){
    switch (key){
        case XK_Escape:
            done = True;
            break;
        case XK_F1:
            killGLWindow();
            GLWin.fs = !GLWin.fs;
            createGLWindow("Electric field", 480, 270, 24, GLWin.fs);
            break;
        case XK_F2:
            ScreenShot();
            break;
        case XK_space:
            if(animationPause == 0){
              animationPause = 1;
            }
            else{
              animationPause = 0;
            }
            break;
        case XK_u:
            translateY += 0.01f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_d:
            translateY -= 0.01f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_l:
            translateX -= 0.01f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_r:
            translateX += 0.01f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_plus:
            translateZ += 0.05f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_minus:
            translateZ -= 0.05f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_Left:
            rotY += 0.25f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_Right:
            rotY -= 0.25f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_Up:
            rotX -= 0.25f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_Down:
            rotX += 0.25f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_Page_Up:
            rotZ += 0.25f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_Page_Down:
            rotZ -= 0.25f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
    }
}
int main(int argc, char **argv){
    XEvent event;
    KeySym key;
    GLfloat steps = 200.0;
    double x, y = 1;
    int i;
    
    animation = 200;
    //savePics = 1;
    startPicNumber = 1000000;
    slowDownMS = 0;

    
    
    
    printf("animation set to %d\n",animation);
    for(x=0; x<10;x++){
      printf("atan2(%lf, %lf)=%lf,    atan2(%lf, %lf)=%lf\n", y, x, atan2(y, x), x, y, atan2(x, y));
    }
        
    done = False;
    /* default to fullscreen */
    GLWin.fs = True;
    if (!createGLWindow("Electric field", 1920, 1080, 24, GLWin.fs)){
        done = True;
    }
    
    
    printf("Force = %lf\n", 8.988f * 1000000000.0f * 0.000001f * 0.000001f / (8.0f * 8.0f));

    
    for(i=0; i<PATHMAX; i++){
      pathWidth[i] = 4.5f;
    }
    
    
    if(animation == 200){
      fieldLineEndUp = 0.0f;

      particles[0][0] = -2.2;
      particles[0][1] = 0.0;
      particles[0][2] = 0.0;
      particles[0][3] = 0.5;
      particles[0][4] = 0.000001;
      
      particles[1][0] = 2.2;
      particles[1][1] = 0.0;
      particles[1][2] = 0.0;
      particles[1][3] = 0.5;
      particles[1][4] = -0.000001;
      
      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -13.3f;
      rotX = 0.0f;
      rotZ = 0.0f;
      rotY = 0.0f;
      
      translateXtarget = 0.0f;
      translateYtarget = 0.0f;
      translateZtarget = -13.3f;
      
      rotXtarget = 0.0f;
      rotZtarget = 0.0f;
      rotYtarget = 0.0f;

      translateXdiff = (translateXtarget - translateX) / steps;
      translateYdiff = (translateYtarget - translateY) / steps;
      translateZdiff = (translateZtarget - translateZ) / steps;
      rotXdiff = (rotXtarget - rotX) / steps;
      rotYdiff = (rotYtarget - rotY) / steps;
      rotZdiff = (rotZtarget - rotZ) / steps;
      
    }



    
    /* wait for events*/ 
    while (!done){
        /* handle the events in the queue */
        while (XPending(GLWin.dpy) > 0){
            XNextEvent(GLWin.dpy, &event);
            switch (event.type){
                case Expose:
	                if (event.xexpose.count != 0)
	                    break;
                    drawGLScene();
                    break;
                case ConfigureNotify:
                /* call resizeGLScene only if our window-size changed */
                    if ((event.xconfigure.width != GLWin.width) || (event.xconfigure.height != GLWin.height)){
                        GLWin.width = event.xconfigure.width;
                        GLWin.height = event.xconfigure.height;
                        printf("Resize event\n");
                        resizeGLScene(event.xconfigure.width,
                            event.xconfigure.height);
                    }
                    break;
                /* exit in case of a mouse button press */
                case ButtonPress:     
                    done = True;
                    break;
                case KeyPress:
                    key = XLookupKeysym(&event.xkey, 0);
                    keyPressed(key);
                    break;
                case ClientMessage:    
                    if (*XGetAtomName(GLWin.dpy, event.xclient.message_type) == *"WM_PROTOCOLS"){
                        printf("Exiting sanely...\n");
                        done = True;
                    }
                    break;
                default:
                    break;
            }
        }
        drawGLScene();
    }
    killGLWindow();
    return 0;
}
